var self = require("sdk/self");
var tabs = require("sdk/tabs");
const contextMenu = require('sdk/context-menu');
var notifications = require("sdk/notifications");
var {ToggleButton} = require('sdk/ui/button/toggle');
var {indexedDB, IDBKeyRange} = require('sdk/indexed-db');
const {Cc, Ci, Cu} = require("chrome");
var clipboard = require("sdk/clipboard");
// const {XMLHttpRequest} = require("sdk/net/xhr");
// var Request = require("sdk/request").Request;
var lsepref = require("sdk/simple-prefs").prefs;
var LSE = require("./js/LSE.js").LSE();
Cu.import("resource://gre/modules/Downloads.jsm");
Cu.import("resource://gre/modules/Task.jsm");
let {getActiveView}=require("sdk/view/core");

//my var
var selectedLinks = "";
const dbName = "lseDatabase";
const dbTable = "lseLinks";
var firstLoad = true;
var torrentDL = false;
var badgeCount = 0;
//my var

tabs.on("ready", followTab);

var downloadedTorrent = function () {
    var view = {
        onDownloadChanged: function (download) {

            if (download.progress == 100 && download.stopped == true && download.canceled == false) {
                var ext = getExtension(download.target.path);
                if (ext.toLowerCase() == "torrent" && lsepref["fetchtorrent"] == 1) {
                    LSE.torrentUpload(download.target.path, dashboard);
                }
            }
        }
    };
    var list;
    Task.spawn(function () {
        list = yield Downloads.getList(Downloads.ALL);
        list.addView(view);
    }).then(null, Cu.reportError);
};

function addItem(item) {
    console.log("lselog: ", "in add");
    var dlnow = false;
    var regen = false;
    if (item.startnow) {
        delete item.startnow;
        dlnow = true;
    }
    if (item.regen) {
        regen = item.regen;
        delete item.regen;

    }
    var transaction = lseDB.transaction([dbTable], "readwrite");
    var store = transaction.objectStore(dbTable);

    var update = store.add(item);

    update.onerror = function (e) {
        console.log("Error", e.target.error.name);
        //some type of error handler
    };

    update.onsuccess = function (e) {
        var nfo = {link: item.link, id: e.target.result};
        if (dlnow) {
            LSE.linkgen(nfo, updateItem, dashboard, true);
        }
        if (regen) {
            nfo.regen = regen;
            LSE.linkgen(nfo, updateItem, dashboard);
        } else {
            dashboard.port.emit("link", nfo);
        }

        console.log("lselog: ", "Added to db");
    };

}
function viewItem(filter, myFunc) {
    console.log("in view");
    var transaction = lseDB.transaction([dbTable], "readwrite");
    var store = transaction.objectStore(dbTable);

    var range = IDBKeyRange.only(filter);
    var index = store.index("status");
    var cursor = index.openCursor(range);
    cursor.onsuccess = myFunc;
}
function deleteItem(thisId) {
    var transaction = lseDB.transaction([dbTable], "readwrite");
    var store = transaction.objectStore(dbTable);
    var request = store.delete(parseInt(thisId));
    request.onsuccess = function () {
        console.log('Deleted that key');
    };
}
function updateItem(item) {
    console.log("lselog: ", "in update");
    console.dir(item);

    var regen = false;

    if (item.regen) {
        regen = item.regen;
        delete item.regen;
    }

    var transaction = lseDB.transaction([dbTable], "readwrite");
    var store = transaction.objectStore(dbTable);

    var update = store.put(item);

    update.onerror = function (e) {
        console.log("Error", e.target.error.name);
        //some type of error handler
    };

    update.onsuccess = function (e) {
        if (!regen) {
            dashboard.port.emit("removeEle", "#" + item.id);
            updateBadge(-1);
        }else{
            dashboard.port.emit("removeEle", "#" + regen);
            deleteItem(regen);
        }
        dashboard.port.emit("generatedLinks", item);

        console.log("lselog: ", "updated the db");
    };
}
function connectDB() {

    var request = indexedDB.open(dbName, 1);

    request.onsuccess = function (event) {
        lseDB = event.target.result;
        console.log("lselog: ", "DB Success");
    };
    request.onblocked = function (event) {
        console.log("lselog: ", "DB Blocked");
    };
    request.onerror = function (event) {
        console.log("lselog: ", "DB Error");
    };
    request.onupgradeneeded = function (event) {
        console.log("lselog: ", "Init New DB");
        lseDB = event.target.result;
        var objectStore;
        if (!lseDB.objectStoreNames.contains(dbTable)) {
            objectStore = lseDB.createObjectStore(dbTable, {keyPath: "id", autoIncrement: true});
            objectStore.createIndex("link", "link", {unique: false});
            objectStore.createIndex("filename", "filename", {unique: false});
            objectStore.createIndex("downloadlink", "downloadlink", {unique: false});
            objectStore.createIndex("status", "status", {unique: false});
        }


        objectStore.transaction.oncomplete = function (event) {
            console.log("lselog: ", "Init Done...");


        };
    };
}

connectDB();

function followTab(tab) {
    if (lsepref["monitortab"] != 1) {
        return false;
    }
    var url = LSE.checkLink(tab.url);
    if (url && url[0].substr(0, 4) == "http") {
        processLink(url[0]);
    }
}

function processLink(url, startnow) {
    var link = {link: url, filename: false, downloadlink: false, status: "pending"};
    if (startnow) {
        link.startnow = true;
    }
    addItem(link);
    updateBadge(1);
    if (!dashboard.isShowing && lsepref["displaypanel"] == 1 && !startnow) {
        dashboard.show();
    }
}


function checkSelect(context) {
    selectedLinks = "";
    if (context.selectionText) {
        selectedLinks = context.selectionText;
    } else {
        selectedLinks = context.linkURL;
    }
    return true;
}

function getExtension(filepath) {
    filepath = filepath.toLowerCase();
    var fileinfo = filepath.split(".");
    return fileinfo[fileinfo.length - 1];
}
function updateBadge(count) {
    badgeCount = badgeCount + count;
    lmlButton.badge = badgeCount;
    if (badgeCount <= 0) {
        lmlButton.badge = "";
    }
    dashboard.port.emit("updateBadge", badgeCount);
    console.log("Badge: " + badgeCount);
}

var menuItem = contextMenu.Item({
    label: "Download with LinkSnappy",
    image: self.data.url("images/icon128.png"),
    context: contextMenu.PredicateContext(checkSelect),
    contentScript: 'self.on("click", function () {self.postMessage("rocket");});',
    onMessage: function (selectionText) {
        //console.log("lse " + selectedLinks);
        if (selectedLinks.substr(0, 8) == "magnet:?") {
            LSE.torrentMagnet(selectedLinks, dashboard, lsepref);
            return;
        }

        var links = LSE.checkLink(selectedLinks);
        if (links) {
            for (var i = 0; i < links.length; i++) {
                if (links[i].substr(0, 4) == "http") {
                    if (lsepref['autosingledl'] && links.length == 1) {
                        processLink(links[i], true);
                    } else {
                        processLink(links[i]);
                    }

                }

            }
        }
        else{
            LSE.notify("Doesn't seem like a filehost link, please download the torrent file to auto capture it");
        }
    }
});

var lmlButton = ToggleButton({
    id: "LinkSnappy",
    label: "LinkSnappy Firefox Plugin",
    icon: {
        "16": self.data.url("images/icon100.png"),
        "32": self.data.url("images/icon100.png"),
        "64": self.data.url("images/icon100.png")
    },
    badge: "",
    badgeColor: "#00AAAA",
    onChange: handleChange
});


var dashboard = require("sdk/panel").Panel({
    width: 400,
    height: 400,
    position: lmlButton,
    contentURL: self.data.url("dash.html"),
    onHide: function () {
        lmlButton.state('window', {checked: false});
    },
    onShow: function () {
        if (firstLoad) {
            badgeCount = 0;
            dashboard.port.emit("init", "linkgen");
            dashboard.port.emit("triggerEvent", {div: ".menuItem .fa-user-secret", event: "click"});
            firstLoad = false;
        }
    },
    contentScriptFile: [self.data.url("js/jquery.min.js"),self.data.url("js/jquery.mCustomScrollbar.concat.min.js"), self.data.url("js/dash.js")]
});


if (lsepref["forcepanel"] == 1) {
    getActiveView(dashboard).setAttribute("noautohide", true);
    getActiveView(dashboard).setAttribute("backdrag", true);
}
function initPending(e) {
    var res = e.target.result;
    if (res) {
        var linkInfo = res.value;
        var nfo = {link: linkInfo.link, id: linkInfo.id};
        dashboard.port.emit("link", nfo);
        updateBadge(1);
        console.dir(linkInfo);
        res.continue();
    }
}

function initGenerated(e) {
    var res = e.target.result;
    if (res) {
        dashboard.port.emit("generatedLinks", res.value);
        res.continue();
    }
}

//init some func
dashboard.port.on("delete", function (info) {
    deleteItem(info);
    dashboard.port.emit("removeEle", "#" + info);
    updateBadge(-1);
    return true;
});
dashboard.port.on("deleteGen", function (info) {
    deleteItem(info);
    dashboard.port.emit("removeEle", "#" + info);
});
dashboard.port.on("getGenerated", function (info) {
    viewItem("generated", initGenerated);
    return true;
});
dashboard.port.on("initPending", function (info) {
    viewItem("pending", initPending);
    return true;
});
dashboard.port.on("fetchtab", function (info) {
    var url = LSE.checkLink(tabs.activeTab.url);
    if (url && url[0].substr(0, 4) == "http") {
        processLink(url[0]);
    }
});
dashboard.port.on("doGenerate", function (info) {
    LSE.linkgen(info, updateItem, dashboard);
    return true;
});
dashboard.port.on("reGenerate", function (info) {
    var link = {link: info.link, filename: false, downloadlink: false, regen: info.id, status: "pending"};
    addItem(link);

});
dashboard.port.on("torrentShow", function (info) {
    LSE.findTorrents(dashboard);
});
dashboard.port.on("getTorrentStats", function (info) {
    LSE.torrentStatus(info, dashboard);
});
dashboard.port.on("startTorrent", function (info) {
    LSE.torrentStart(info, dashboard);
});
dashboard.port.on("zipTorrent", function (info) {
    LSE.torrentZip(info, dashboard);
});
dashboard.port.on("delTorrent", function (info) {
    LSE.torrentDelete(info, dashboard);
});
dashboard.port.on("getFilehost", function (info) {
    LSE.filehostStats(dashboard);
});
dashboard.port.on("grabAccount", function (info) {
    LSE.accountInfo(dashboard);
});
dashboard.port.on("doLogout", function (info) {
    LSE.logout(dashboard, false);
});
dashboard.port.on("doLogin", function (info) {
    LSE.login(info.username, info.password, dashboard);
});
dashboard.port.on("doRegister", function (email) {
    LSE.register(email, dashboard);
});
dashboard.port.on("notify", function (info) {
    LSE.notify(info.text, info.link);
});
dashboard.port.on("processClip", function (info) {
    var links = LSE.checkLink(clipboard.get("text"));
    if (links) {
        for (var i = 0; i < links.length; i++) {
            if (links[i].substr(0, 4) == "http") {
                processLink(links[i]);
            }
        }
    }

});
dashboard.port.on("clipboard", function (info) {

    switch (info.job) {
        case "append":
            var cData = clipboard.get("text");
            if (cData == null || cData == "") {
                clipboard.set(info.text, "text");
            } else {
                clipboard.set(cData + "\n" + info.text, "text");
            }
            break;
        case "add":
            clipboard.set(info.text, "text");
            break;
        case "clear":
            clipboard.set("", "text");
            break;
    }
});
dashboard.port.on("settings", function (info) {
    console.log(info);
    lsepref[info.setting] = info.value;
    if (info.setting == "forcepanel") {
        if (info.value == 0) {
            getActiveView(dashboard).setAttribute("noautohide", false);
            getActiveView(dashboard).setAttribute("backdrag", false);
        } else {
            getActiveView(dashboard).setAttribute("noautohide", true);
            getActiveView(dashboard).setAttribute("backdrag", true);
        }
    }
});
dashboard.port.on("getSettings", function (info) {
    var settings = ["monitortab", "autodl", "fetchtorrent", "displaypanel", "chkupdate", "forcepanel", "autosingledl"];
    for (var i = 0; i < settings.length; i++) {
        dashboard.port.emit("getSettings", {setting: settings[i], value: lsepref[settings[i]]})
    }
});

dashboard.port.on("stream", function (info) {
    if (dashboard.isShowing) {
        dashboard.hide();
    }
    if (info.torrent) {
        var url = info.link.replace("download", "stream")
        tabs.open({
            url: url
        });
    } else {
        var dlsrv = findStr(info.link, "http://dlserv", ".");
        var linksplit = info.link.split("/");
        var url = "https://linksnappy.com/watch/" + dlsrv + "/" + linksplit[4] + "/" + linksplit[5];
        tabs.open({
            url: url
        });
    }
    // if(info.torrent){
    //   console.log(info.link);
    //   var req = new XMLHttpRequest();
    //   req.open("GET", info.link);
    //   req.onreadystatechange = function(){
    //     if(req.readyState == 2){
    //       var myTab = tabs.activeTab;
    //       myTab.attach({
    //         contentScriptFile: self.data.url("js/stream.js"),
    //         contentScriptOptions: {
    //           link: req.responseURL,
    //           type: info.type
    //         }
    //
    //       });
    //       req.close();
    //     }
    //   };
    //   req.send();
    //   return true;
    // }
    //   var myTab = tabs.activeTab;
    //   myTab.attach({
    //       contentScriptFile: self.data.url("js/stream.js"),
    //       contentScriptOptions: {
    //           link: info.link,
    //           type: info.type
    //       }
    //
    //   });
});
dashboard.port.on("rebuildregex", function (url) {
    LSE.getRegex(true);
});
dashboard.port.on("reportlink", function (info) {
    LSE.reportlink(info.link, info.genlink, info.msg);
});
dashboard.port.on("openlink", function (url) {
    tabs.open({url: url});
});
dashboard.port.on("twoFactor", function (info) {
    LSE.twoFactor(info.reqCode, info.twofpin, dashboard);
});

dashboard.port.on("browseTorrent", function (info) {
    LSE.browseTorrents(info.link, dashboard);
});
dashboard.port.on("badgeClear", function (info) {
    lmlButton.badge = "";
});
if (lsepref["chkupdate"] == 1) {
    LSE.checkVersion();
}
if (lsepref["fetchtorrent"] != 0) {
    torrentDL = new downloadedTorrent();
}


function initPref() {
    var settings = ["monitortab", "autodl", "fetchtorrent", "displaypanel", "chkupdate", "forcepanel", "autosingledl"];
    for (var i = 0; i < settings.length; i++) {
        if (lsepref[settings[i]] == undefined) {
            lsepref[settings[i]] = 1;
            if (settings[i] == "forcepanel") {
                lsepref[settings[i]] = 0;
                getActiveView(dashboard).setAttribute("noautohide", false);
                getActiveView(dashboard).setAttribute("backdrag", false);
            }
            if (settings[i] == "autosingledl") {
                lsepref[settings[i]] = 0;
            }
        }
    }
}


initPref();

function handleChange(state) {
    if (state.checked) {
        dashboard.show();
    } else {
        dashboard.hide();
    }
}


function findStr(str, start, end) {
    var startPoint = str.indexOf(start) + start.length;
    var endPoint = str.indexOf(end);
    if (Number.isInteger(startPoint) && Number.isInteger(endPoint)) {
        str = str.substr(startPoint);
        endPoint = str.indexOf(end);
        str = str.substr(0, endPoint);
        return str;
    } else {
        return false;
    }
}

